############################################################################
#                                                                          #
# Determine how to invest excess cash in 1-month, 3-month and 6-month CDs  #
# so as to maximize interest income while meeting company cash             #
# requirements (including a margin of safety).                             #
#                                                                          #
############################################################################

set CDS;
set MONTHS ordered;
set BUYING within {CDS,MONTHS};

##############
# Parameters #
##############

param INITIAL_CASH;                       # Initial amount of money
param MONTHLY_CASH_USE {MONTHS};          # Cash outflow each month
param CD_YIELD {CDS};                     # Yield of CDs
param CD_TERM {CDS};                      # Term of CDS
param CD_PRICE {CDS};                     # Price of CDs
param SAFETY;                             # Minimum amount left each month

#############
# Variables #
#############

var CDS_PURCHASED {BUYING} integer >=0;   # Number of CDs to buy 
var MONTHLY_CD_INFLOW {MONTHS};           # Monthly interest and maturing CDs
var MONTHLY_BALANCE {MONTHS};             # Money left over each month

#########
# Model #
#########

maximize TOTAL_INTEREST:
    sum {(i,j) in BUYING} CDS_PURCHASED[i,j]*CD_PRICE[i]*CD_YIELD[i];
subject to
   Monthly_inflow {j in MONTHS}:
   MONTHLY_CD_INFLOW[j] = 
      sum {i in CDS:ord(j) > CD_TERM[i]} CD_PRICE[i]*(1+CD_YIELD[i])*(
                   if (i,prev(j,CD_TERM[i])) in BUYING then
                      CDS_PURCHASED[i,prev(j,CD_TERM[i])]
                   else 0); 
   Left_each_month {j in MONTHS}:    
   MONTHLY_BALANCE[j] = if j=first(MONTHS) 
           then 
              INITIAL_CASH - MONTHLY_CASH_USE[j]
              - sum {i in CDS} CDS_PURCHASED[i,j]*CD_PRICE[i] 
           else  
              MONTHLY_BALANCE[prev(j)] +MONTHLY_CD_INFLOW[j]-MONTHLY_CASH_USE[j] 
              -sum {i in CDS:(i,j) in BUYING} CDS_PURCHASED[i,j]*CD_PRICE[i];
  Safety_margin {j in MONTHS}:
  MONTHLY_BALANCE[j] >= SAFETY;
