##########################################################################
#                                                                        #
# A company wants to reorganize its labor force. It currently has three  #
# different kind of employees; untrained, moderately trained and highly  #
# trained. Over the coming years, the company expects a necessary shift  #
# to more trained employees. How should the company reorganize to        #
# minimize cost? Or minimize the number of employees that have to be     #
# laid off?                                                              #
#                                                                        #
##########################################################################

set EMPLOYEES ordered;
set YEARS ordered;

##############
# Parameters #
##############

param LAY_OFF_COST  {EMPLOYEES};               # Cost to lay off employees
param AVAILABLE_TO_HIRE {EMPLOYEES};           # People available to hire
param HIRING_COST {EMPLOYEES};                 # Cost to hire employees 
param CURRENT_EMPLOYEES {EMPLOYEES};           # Number of current employees
param EMPLOYEES_REQUIRED {EMPLOYEES,YEARS};    # Employees required in future
param MOD_TRAINING_COST;                       # Cost to train low-skilled employees
param HIGH_TRAINING_COST;                      # Cost to train moderately skilled employees

#############
# Variables #
#############

var EMPLOYEES_MOD_TRAIN {YEARS} >=0;           # Low skilled people to train
var EMPLOYEES_HIGH_TRAIN {YEARS} >=0;          # Moderately skilled people to train
var EMPLOYEES_HIRED {EMPLOYEES,YEARS} >=0;     # Employees to hire
var EMPLOYEES_LAID_OFF {EMPLOYEES,YEARS} >=0;  # Employees to lay off

#########
# Model #
#########

minimize TOTAL_COST:
   sum {j in YEARS} (EMPLOYEES_MOD_TRAIN[j]*MOD_TRAINING_COST+
                     EMPLOYEES_HIGH_TRAIN[j]*HIGH_TRAINING_COST)+
   sum {i in EMPLOYEES, j in YEARS} (EMPLOYEES_HIRED[i,j]*HIRING_COST[i]+
                                     EMPLOYEES_LAID_OFF[i,j]*LAY_OFF_COST[i]);
subject to
   Training_of_low_skilled {j in YEARS, i in EMPLOYEES:ord(i)=1}:
      EMPLOYEES_MOD_TRAIN[j] <= if ord(j)=1 then CURRENT_EMPLOYEES[i]
                                else EMPLOYEES_REQUIRED[i,prev(j)];
   Training_of_moderately_skilled {i in EMPLOYEES,j in YEARS:ord(i)=2}:
     EMPLOYEES_HIGH_TRAIN[j] <= if ord(j)=1 then CURRENT_EMPLOYEES[i]
                                else EMPLOYEES_REQUIRED[i,prev(j)];
   Hiring {i in EMPLOYEES,j in YEARS}:
      EMPLOYEES_HIRED[i,j] <= AVAILABLE_TO_HIRE[i];
   Firing {i in EMPLOYEES, j in YEARS}:
      EMPLOYEES_LAID_OFF[i,j] <= if ord(j)=1 then CURRENT_EMPLOYEES[i]
                                 else EMPLOYEES_REQUIRED[i,prev(j)];
   Low_skill_requirement {i in EMPLOYEES,j in YEARS: ord(i)=1}:
      if ord(j)=1 then 
         CURRENT_EMPLOYEES[i]+EMPLOYEES_HIRED[i,j]-
         EMPLOYEES_MOD_TRAIN[j]-EMPLOYEES_LAID_OFF[i,j]
      else
         EMPLOYEES_REQUIRED[i,prev(j)]+EMPLOYEES_HIRED[i,j]-
         EMPLOYEES_MOD_TRAIN[j]-EMPLOYEES_LAID_OFF[i,j]
         = EMPLOYEES_REQUIRED[i,j];
   Moderate_skill_requirement {i in EMPLOYEES,j in YEARS:ord(i)=2}:
      if ord(j)=1 then 
         CURRENT_EMPLOYEES[i]+EMPLOYEES_HIRED[i,j]-
         EMPLOYEES_HIGH_TRAIN[j]-EMPLOYEES_LAID_OFF[i,j]+
         EMPLOYEES_MOD_TRAIN[j]
      else
         EMPLOYEES_REQUIRED[i,prev(j)]+EMPLOYEES_HIRED[i,j]+
         EMPLOYEES_MOD_TRAIN[j]-EMPLOYEES_LAID_OFF[i,j]-
         EMPLOYEES_HIGH_TRAIN[j]
         = EMPLOYEES_REQUIRED[i,j];
   Highly_skilled_requirement {i in EMPLOYEES, j in YEARS:ord(i)=3}:   
      if ord(j)=1 then
         CURRENT_EMPLOYEES[i]+EMPLOYEES_HIRED[i,j]+
         EMPLOYEES_LAID_OFF[i,j]+EMPLOYEES_HIGH_TRAIN[j]
      else
         EMPLOYEES_REQUIRED[i,prev(j)]+EMPLOYEES_HIRED[i,j]+
         EMPLOYEES_LAID_OFF[i,j]+EMPLOYEES_HIGH_TRAIN[j]
         = EMPLOYEES_REQUIRED[i,j];
# The following constraint is optional: if the desire is to have nobody        
# laid off, include this constraint
#   Nobody_gets_laid_off:
#    sum {i in EMPLOYEES, j in YEARS} EMPLOYEES_LAID_OFF[i,j] = 0;

